==> retro_data_structures-0.26.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/PuddleControlPhaseData.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class PuddleControlPhaseDataJson(typing_extensions.TypedDict):
        generation_rate: float
        effect_rate: float
        move_rate: float
        duration: float
    

_FAST_FORMAT = None
_FAST_IDS = (0x23e91e15, 0x82403cbd, 0x7ec9e6d9, 0x8b51e23f)


@dataclasses.dataclass()
class PuddleControlPhaseData(BaseProperty):
    generation_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x23e91e15, original_name='GenerationRate'
        ),
    })
    effect_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x82403cbd, original_name='EffectRate'
        ),
    })
    move_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x7ec9e6d9, original_name='MoveRate'
        ),
    })
    duration: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x8b51e23f, original_name='Duration'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 4:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHfLHfLHf')
    
        dec = _FAST_FORMAT.unpack(data.read(40))
        assert (dec[0], dec[3], dec[6], dec[9]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
            dec[8],
            dec[11],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x04')  # 4 properties

        data.write(b'#\xe9\x1e\x15')  # 0x23e91e15
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.generation_rate))

        data.write(b'\x82@<\xbd')  # 0x82403cbd
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.effect_rate))

        data.write(b'~\xc9\xe6\xd9')  # 0x7ec9e6d9
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.move_rate))

        data.write(b'\x8bQ\xe2?')  # 0x8b51e23f
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.duration))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("PuddleControlPhaseDataJson", data)
        return cls(
            generation_rate=json_data['generation_rate'],
            effect_rate=json_data['effect_rate'],
            move_rate=json_data['move_rate'],
            duration=json_data['duration'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'generation_rate': self.generation_rate,
            'effect_rate': self.effect_rate,
            'move_rate': self.move_rate,
            'duration': self.duration,
        }


def _decode_generation_rate(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_effect_rate(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_move_rate(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


def _decode_duration(data: typing.BinaryIO, property_size: int):
    return struct.unpack('>f', data.read(4))[0]


_property_decoder: typing.Dict[int, typing.Tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x23e91e15: ('generation_rate', _decode_generation_rate),
    0x82403cbd: ('effect_rate', _decode_effect_rate),
    0x7ec9e6d9: ('move_rate', _decode_move_rate),
    0x8b51e23f: ('duration', _decode_duration),
}

==> retro_data_structures-0.29.0-py3-none-any.whl: retro_data_structures/properties/corruption/archetypes/PuddleControlPhaseData.py <==
# Generated File
from __future__ import annotations

import dataclasses
import struct
import typing
import typing_extensions

from retro_data_structures import json_util
from retro_data_structures.game_check import Game
from retro_data_structures.properties.base_property import BaseProperty
from retro_data_structures.properties.field_reflection import FieldReflection

if typing.TYPE_CHECKING:
    class PuddleControlPhaseDataJson(typing_extensions.TypedDict):
        generation_rate: float
        effect_rate: float
        move_rate: float
        duration: float
    

_FAST_FORMAT: struct.Struct | None = None
_FAST_IDS = (0x23e91e15, 0x82403cbd, 0x7ec9e6d9, 0x8b51e23f)


@dataclasses.dataclass()
class PuddleControlPhaseData(BaseProperty):
    generation_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x23e91e15, original_name='GenerationRate'
        ),
    })
    effect_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x82403cbd, original_name='EffectRate'
        ),
    })
    move_rate: float = dataclasses.field(default=1.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x7ec9e6d9, original_name='MoveRate'
        ),
    })
    duration: float = dataclasses.field(default=0.0, metadata={
        'reflection': FieldReflection[float](
            float, id=0x8b51e23f, original_name='Duration'
        ),
    })

    @classmethod
    def game(cls) -> Game:
        return Game.CORRUPTION

    @classmethod
    def from_stream(cls, data: typing.BinaryIO, size: int | None = None, default_override: dict | None = None) -> typing_extensions.Self:
        property_count = struct.unpack(">H", data.read(2))[0]
        if (result := cls._fast_decode(data, property_count)) is not None:
            return result

        present_fields = default_override or {}
        for _ in range(property_count):
            property_id, property_size = struct.unpack(">LH", data.read(6))
            start = data.tell()
            try:
                property_name, decoder = _property_decoder[property_id]
                present_fields[property_name] = decoder(data, property_size)
            except KeyError:
                raise RuntimeError(f"Unknown property: 0x{property_id:08x}")
            assert data.tell() - start == property_size

        return cls(**present_fields)

    @classmethod
    def _fast_decode(cls, data: typing.BinaryIO, property_count: int) -> typing_extensions.Self | None:
        if property_count != 4:
            return None
    
        global _FAST_FORMAT
        if _FAST_FORMAT is None:
            _FAST_FORMAT = struct.Struct('>LHfLHfLHfLHf')
    
        dec = _FAST_FORMAT.unpack(data.read(40))
        assert (dec[0], dec[3], dec[6], dec[9]) == _FAST_IDS
        return cls(
            dec[2],
            dec[5],
            dec[8],
            dec[11],
        )

    def to_stream(self, data: typing.BinaryIO, default_override: dict | None = None) -> None:
        default_override = default_override or {}
        data.write(b'\x00\x04')  # 4 properties

        data.write(b'#\xe9\x1e\x15')  # 0x23e91e15
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.generation_rate))

        data.write(b'\x82@<\xbd')  # 0x82403cbd
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.effect_rate))

        data.write(b'~\xc9\xe6\xd9')  # 0x7ec9e6d9
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.move_rate))

        data.write(b'\x8bQ\xe2?')  # 0x8b51e23f
        data.write(b'\x00\x04')  # size
        data.write(struct.pack('>f', self.duration))

    @classmethod
    def from_json(cls, data: json_util.JsonValue) -> typing_extensions.Self:
        json_data = typing.cast("PuddleControlPhaseDataJson", data)
        return cls(
            generation_rate=json_data['generation_rate'],
            effect_rate=json_data['effect_rate'],
            move_rate=json_data['move_rate'],
            duration=json_data['duration'],
        )

    def to_json(self) -> json_util.JsonObject:
        return {
            'generation_rate': self.generation_rate,
            'effect_rate': self.effect_rate,
            'move_rate': self.move_rate,
            'duration': self.duration,
        }


def _decode_generation_rate(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_effect_rate(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_move_rate(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


def _decode_duration(data: typing.BinaryIO, property_size: int) -> float:
    return struct.unpack('>f', data.read(4))[0]


_property_decoder: dict[int, tuple[str, typing.Callable[[typing.BinaryIO, int], typing.Any]]] = {
    0x23e91e15: ('generation_rate', _decode_generation_rate),
    0x82403cbd: ('effect_rate', _decode_effect_rate),
    0x7ec9e6d9: ('move_rate', _decode_move_rate),
    0x8b51e23f: ('duration', _decode_duration),
}
